package org.average.nfcauthcr;

import android.app.Service;
import android.app.PendingIntent;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.TagLostException;
import android.nfc.tech.IsoDep;
import android.os.Binder;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;
import android.widget.Toast;

import org.average.nfcauthcr.CheckConnector;
import org.average.nfcauthcr.YkNeo;
import org.average.nfcauthcr.CRException;

public class Check extends Service {

	private final String TAG = getClass().getName();

	private final CheckConnector checkConnector = new CheckConnector();
	private SharedPreferences prefs;
	private int startId;
	private PendingIntent tagIntent;

	@Override
	public void onCreate() {
		Log.v(TAG, "Created");
		prefs = PreferenceManager.getDefaultSharedPreferences(this);
	}

	@Override
	public int onStartCommand (Intent intent, int flags, int startId) {
		Log.v(TAG, "Starting service");
		this.startId = startId;
		int slot = prefs.getInt("slot_number", -1);
		if (slot > 0) {
			enableDispatch(slot);
		} else {
			stopSelf(startId);
		}
		return START_NOT_STICKY;
	}

	@Override
	public IBinder onBind(Intent intent) {
		Log.v(TAG, "Binding");
		checkConnector.setService(this);
		return checkConnector;
	}

	@Override
	public void onDestroy() {
		Log.v(TAG, "Finished the run");
		disableDispatch();
	}

	@Override
	protected void onNewIntent(Intent intent) {
		Log.v(TAG, "NFC Intent arrived");
		int slot = intent.getIntExtra("slot", -1);
		byte[] challenge = intent.getByteArrayExtra("challenge");
		if (slot <= 0) return;
		Tag tag = intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
		if (tag == null) return;
		IsoDep isoTag = IsoDep.get(tag);
		try {
			byte[] response = YkNeo.doChallengeYubiKey(
						isoTag, slot, challenge);
			Intent data = getIntent();
			data.putExtra("response", response);
			setResult(RESULT_OK, data);
		} catch (CRException e) {
			Log.v(TAG, e.getMessage());
			Toast.makeText(this, e.getMessage(),
					Toast.LENGTH_LONG).show();
		}
		finish();
	}

	private void enableDispatch(int slot) {
		Intent intent = getIntent();
		intent.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP);
		intent.putExtra("slot", slot);
		tagIntent = PendingIntent.getActivity(this, 0, intent, 0);
		IntentFilter iso =
			new IntentFilter(NfcAdapter.ACTION_TECH_DISCOVERED);
		NfcAdapter adapter = NfcAdapter.getDefaultAdapter(this);
		if (adapter == null) {
			Toast.makeText(this, R.string.no_nfc,
						Toast.LENGTH_LONG).show();
			finish();
		}
		if (! adapter.isEnabled()) {
			Toast.makeText(this, R.string.nfc_disabled,
						Toast.LENGTH_LONG).show();
			finish();
		}
		adapter.enableForegroundDispatch(
			this, tagIntent, new IntentFilter[] {iso},
			new String[][] {new String[] {IsoDep.class.getName()}});
	}

	private void disableDispatch() {
		if (tagIntent != null) {
			tagIntent.cancel();
			tagIntent = null;
		}
		NfcAdapter adapter = NfcAdapter.getDefaultAdapter(this);
		if (adapter != null) {
			adapter.disableForegroundDispatch(this);
		}
	}
}
